//-*-c++-*------------------------------------------------------------
//
// File name : bioArithPrint.cc
// Author :    Michel Bierlaire
// Date :      Tue May 11 07:10:16 2010
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patError.h"
#include "patFileNames.h"
#include "bioVersion.h"
#include "bioRowIterator.h"
#include "bioArithPrint.h"
#include "bioArithBinaryPlus.h"
#include "bioLiteralRepository.h"
#include "bioIteratorInfoRepository.h"
#include "bioExpressionRepository.h"
#include "bioSample.h"
#include "patHybridMatrix.h"
#include "patMultivariateNormal.h"
#include "bioParameters.h"
#include "bioRandomDraws.h"
#include "patNormalWichura.h"
#include "bioSensitivityAnalysis.h"

/*!
*/
bioArithPrint::bioArithPrint(bioExpressionRepository* rep,
			     patULong par,
			     map<patString,patULong> theTerms,
			     patString it,patError*& err) 
  : bioExpression(rep, par),theIteratorName(it),theExpressionsId(theTerms), theRandomDraws(NULL) {
  
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  bioIteratorSpan theSpan(it,0) ;
  setCurrentSpan(theSpan) ;
  theIteratorType = bioIteratorInfoRepository::the()->getType(theIteratorName,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  for (map<patString,patULong>::iterator i = theExpressionsId.begin() ;
       i != theExpressionsId.end() ;
       ++i) {
    theExpressions[i->first] = theRepository->getExpression(i->second) ;
  }

}

bioArithPrint::~bioArithPrint() {}

patString bioArithPrint::getOperatorName() const {
  return patString("Print") ;
}

bioExpression* bioArithPrint::getDerivative(patULong aLiteralId, patError*& err) const {

  err = new patErrMiscError("No derivative for the print expression") ;
  WARNING(err->describe()) ;
  return NULL;
}


bioArithPrint* bioArithPrint::getDeepCopy(bioExpressionRepository* rep,
					  patError*& err) const {

  map<patString,patULong> newDict ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    bioExpression* clone = i->second->getDeepCopy(rep,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    newDict[i->first] = clone->getId() ;
  }

  bioArithPrint* theNode = 
    new bioArithPrint(rep,patBadId,newDict,theIteratorName,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  
  return theNode ;
}

bioArithPrint* bioArithPrint::getShallowCopy(bioExpressionRepository* rep,
					  patError*& err) const {

  bioArithPrint* theNode = 
    new bioArithPrint(rep,patBadId,theExpressionsId,theIteratorName,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  
  return theNode ;
}



patBoolean bioArithPrint::isSum() const {
  return patFALSE ;
}

patBoolean bioArithPrint::isProd() const {
  return patFALSE ;
}


patString bioArithPrint::getExpression(patError*& err) const {
  stringstream ss ;
  ss << bioIteratorInfoRepository::the()->getInfo(theIteratorName,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString() ;
  }
  ss << "[" ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i != theExpressions.begin()) {
      ss << ", " ;
    }
    ss << i->first << "=" << *(i->second) ;
  }
  ss << "]" ;

  return patString(ss.str()) ;
  
}

patULong bioArithPrint::getNumberOfOperations() const {
  patULong count = 0 ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    count += i->second->getNumberOfOperations() ;
  }
  return(count) ;

}

patString bioArithPrint::getCppCode(patString prefix, patError*& err) {
  err = new patErrMiscError("Not implemented") ;
  WARNING(err->describe()) ;
  return patString() ;
}

void bioArithPrint::generateGslFunctions(ostream& str_h,patString prefix,patError*& err) {
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    
    i->second->generateGslFunctions(str_h,prefix,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
  }

}

patReal bioArithPrint::getValue(patBoolean prepareGradient,
				patError*& err)  {
  
  err = new patErrMiscError("Irrelevant with this object") ;
  WARNING(err->describe()) ;
  return patReal() ;
}


bioFunctionAndDerivatives* bioArithPrint::getNumericalFunctionAndGradient(vector<patULong> literalIds, patBoolean computeHessian, patError*& err) {
  stringstream str ;
  str << "Not implemented for expression " << getOperatorName() ;
  err = new patErrMiscError(str.str()) ;
  WARNING(err->describe()) ;
  return NULL ;
  
}

void bioArithPrint::setVariables(const patVariables* x) {
  __x = x ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->setVariables(x) ;
  }
}
//void bioArithPrint::setDraws(const vector< vector<patReal> >* d) {
void bioArithPrint::setDraws(patReal** d) {
  __draws = d ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->setDraws(d) ;
  }
}

void bioArithPrint::setSample(bioSample* s) {
  theSample = s ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->setSample(s) ;
  }
}

void bioArithPrint::setCurrentSpan(bioIteratorSpan aSpan) {
  theCurrentSpan = aSpan ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->setCurrentSpan(aSpan) ;
  }
}

void bioArithPrint::setThreadSpan(bioIteratorSpan aSpan) {
  theThreadSpan = aSpan ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->setThreadSpan(aSpan) ;
  }
}

patString bioArithPrint::getSimpleCppCode(patError*& err)  {
  stringstream str ;
  str << "Not implemented for expression " << getOperatorName() ;
  err = new patErrMiscError(str.str()) ;
  WARNING(err->describe()) ;
  return patString() ;


}


patBoolean bioArithPrint::dependsOf(patULong aLiteralId) const {
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i->second->dependsOf(aLiteralId)) {
      return patTRUE ;
    }
  }
  return patFALSE ;

}

void bioArithPrint::forceCodeGeneration() {
  codeMustBeGenerated = patTRUE ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->forceCodeGeneration() ;
  }

}

patBoolean bioArithPrint::containsAnIterator() const {
  return patTRUE ;
}

patBoolean bioArithPrint::containsAnIteratorOnRows() const {
  return theIteratorType == ROW ;
}


patBoolean bioArithPrint::containsAnIntegral() const {
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i->second->containsAnIntegral()) {
      return patTRUE ;
    }
  }
  return patFALSE ;
}

patBoolean bioArithPrint::containsASequence() const {
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i->second->containsASequence()) {
      return patTRUE ;
    }
  }
  return patFALSE ;

}


patString bioArithPrint::getIncludeStatements(patString prefix) const {
  return patString("Not implemented") ;
}

void bioArithPrint::simplifyZeros(patError*& err) {
  return ;
}

void bioArithPrint::collectExpressionIds(set<patULong>* s) const {
  s->insert(getId()) ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    i->second->collectExpressionIds(s) ;
  }
}

patString bioArithPrint::getExpressionString() const {
  stringstream str ;
  str << "$Print{" ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i != theExpressions.begin()) {
      str << "," ;
    }
    str << i->second->getExpressionString() ;
  }
  str << "}" ;
  return patString(str.str()) ;
}

patBoolean bioArithPrint::isSimulator() const {
  return patTRUE ;
}

void bioArithPrint::simulate(patHybridMatrix* varCovar, 
			     vector<patString> betaNames, 
			     bioExpression* weight,
			     patError*& err) {

  vector<patVariables> simulatedBetas ;
  patULong nDraws = bioParameters::the()->getValueInt("NbrOfDrawsForSensitivityAnalysis",err)  ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  
  // Prepare the sensitivity analysis

  patVariables mean ;
  if (varCovar != NULL) {
    DEBUG_MESSAGE("Prepare sensitivy analysis for " << betaNames.size() << " parameters") ;
    simulatedParam.resize(betaNames.size()) ;
    for (patULong b = 0 ; b < betaNames.size()  ; ++b) {
      patString name = betaNames[b] ;
      patReal literalId = bioLiteralRepository::the()->getLiteralId(name,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      if (literalId != patBadId) {
	simulatedParam[b] = patTRUE ;
	patReal v = bioLiteralRepository::the()->getBetaValue(name,err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return ;
	}
	mean.push_back(v) ;
      }
      else {
	simulatedParam[b] = patFALSE ;
	mean.push_back(0.0) ;
	stringstream str ;
	str << "Parameter " << name << " is not used in the formula to simulate. It is unnecesary to compute the sensitivity with respect to it." ;
	WARNING(str.str()) ;
      }
    }
    patReal se =  bioParameters::the()->getValueReal("TolSchnabelEskow",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }

    patNormalWichura* theNormal = new patNormalWichura() ;
    theNormal->setUniform(bioRandomDraws::the()->getUniformGenerator()) ;

    ofstream debugFile("beta.draws") ;
    for (vector<patString>::iterator b = betaNames.begin() ;
	 b != betaNames.end() ;
	 ++b) {
      debugFile << *b << '\t' ;
    }
    debugFile << endl ;
    debugFile << mean << endl ;
    debugFile << "+++++++++++++++" << endl ;

    theRandomDraws = new patMultivariateNormal(&mean,
					       varCovar,
					       theNormal,
					       se,
					       err) ;
      
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    for (patULong r = 0 ; r < nDraws ; ++r) {
      patVariables newBeta = theRandomDraws->getNextDraw(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      debugFile << newBeta << endl ;
      simulatedBetas.push_back(newBeta) ;
    }
    debugFile.close() ;
  }
  if (theIteratorType == ROW) {


    bioRowIterator* theIter = theSample->createRowIterator(theCurrentSpan,
							   theThreadSpan,
							   patTRUE,
							   err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }

    patString weightHeader = bioParameters::the()->getValueString("HeaderForWeightInSimulatedResults",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return  ;
    }


    patULong nbrOfRows(0) ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      // First, the weight, if asked
      if (weight != NULL) {
	weight->setVariables(theIter->currentItem()) ;
	patReal w = weight->getValue(patFALSE,err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return  ;
	}
	simulatedValues[weightHeader].push_back(w) ;
	
      }

      bioSensitivityAnalysis theAnalysis ;
      ++nbrOfRows ;
      for (map<patString,bioExpression*>::iterator i = theExpressions.begin() ;
	   i != theExpressions.end() ;
	   ++i) {
	if (varCovar != NULL) {
	  for (patULong k = 0 ; k < betaNames.size() ; ++k) {
	    if (simulatedParam[k]) {
	      bioLiteralRepository::the()->setBetaValue(betaNames[k],mean[k],err) ;
	      if (err != NULL) {
		WARNING(err->describe()) ;
		return ;
	      }
	    }
	  }
	}

	i->second->setVariables(theIter->currentItem()) ;
	patReal res = i->second->getValue(patFALSE,err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return ;
	}
	simulatedValues[i->first].push_back(res) ;

	if (varCovar != NULL) {
	  // Sensitivity analysis
	  for (patULong r = 0 ; r < nDraws ; ++r) { 
	    for (patULong k = 0 ; k < simulatedBetas[r].size() ; ++k) {
	      if (simulatedParam[k]) {
		bioLiteralRepository::the()->setBetaValue(betaNames[k],simulatedBetas[r][k],err) ;
		if (err != NULL) {
		  WARNING(err->describe()) ;
		  return ;
		}
	      }
	    }
	    patReal res = i->second->getValue(patFALSE,err) ;
	    if (err != NULL) {
	      WARNING(err->describe()) ;
	      return ;
	    }
	    stringstream str ;
	    theAnalysis.addData(i->first,res) ;
	  }
	  patReal alpha = bioParameters::the()->getValueReal("sensitivityAnalysisAlpha",err) ;
	  if (err != NULL) {
	    WARNING(err->describe()) ;
	    return ;
	  }
	  vector<patReal> qa  ;
	  qa.push_back(alpha) ;
	  qa.push_back(0.5) ;
	  qa.push_back(1-alpha) ;
	  vector<patReal> quantiles = theAnalysis.getQuantiles(i->first,qa,err) ;
	  if (err != NULL) {
	    WARNING(err->describe()) ;
	    return ;
	  }
	  stringstream q1,q2,qmed ;
	  q1 << i->first << "_" << int(alpha*100) ;
	  simulatedValues[patString(q1.str())].push_back(quantiles[0]) ;
	  qmed << i->first << "_median" ;
	  simulatedValues[patString(qmed.str())].push_back(quantiles[1]) ;
	  q2 << i->first << "_" << int(100*(1-alpha)) ;
	  simulatedValues[patString(q2.str())].push_back(quantiles[2]) ;
	  patULong dump = bioParameters::the()->getValueInt("DumpSensitivityAnalysis",err) ;
	  if (err != NULL) {
	    WARNING(err->describe()) ;
	    return ;
	  }
	  if (dump != 0) {
	    theAnalysis.dumpOnFile(err) ;
	  }
	  if (err != NULL) {
	    WARNING(err->describe()) ;
	    return ;
	  }
	}
      }
    }


    DELETE_PTR(theIter) ;
  }
  else {
    err = new patErrMiscError("Simulation only on row iterators") ;
    WARNING(err->describe()) ;
    return  ;
  }

}

map<patString,vector<patReal> >* bioArithPrint::getSimulationResults()  {
  return &simulatedValues ;
}

patString bioArithPrint::check(patError*& err) const {
  stringstream str ;
  for (map<patString,bioExpression*>::const_iterator  i = theExpressions.begin();
       i != theExpressions.end() ;
       ++i) {
    map<patString,patULong>::const_iterator found = theExpressionsId.find(i->first) ;
    if (found == theExpressionsId.end()) {
      str << "Incompatible data structure for bioArithPrint: expression " << i->first << " does not have an associated ID" << endl ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()) ;
      return patString() ;
    }
    if (i->second->getId() != found->second) {
      str << "Incompatible IDS for children: " << i->second->getId() << " and " << found->second << endl ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()) ;
      return patString() ;
    }
    if (i->second->getParent() != getId()) {
      str << "Incompatible IDS for parent: " << i->second->getParent() << " and " << getId() << endl ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()) ;
      return patString() ;
      
    }
  }
  return patString(str.str()) ;
}
