//-*-c++-*------------------------------------------------------------
//
// File name : bioReporting.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Aug 17 18:47:55 2009
//
//--------------------------------------------------------------------

#ifndef bioReporting_h
#define bioReporting_h

#include "patError.h"
#include "patMyMatrix.h"

class bioRawResults ;
class bioBayesianResults ;
class bioSample ;

class bioReporting {

 public:
  bioReporting() ;
  void writeHtml(patString fileName, patError*& err) ;
  void writeLaTeX(patString fileName, patError*& err) ;
  patBoolean isRaoCramerAvailable() const ;
  patBoolean isSandwichAvailable() const ;
  void computeFromSample(bioSample* s, patError*& err)  ;
  void computeEstimationResults(bioRawResults* rr,patError*& err) ;
  void computeBayesianResults(bioBayesianResults* rr,patError*& err) ;
  void setDiagnostic(patString diag) ;
  void setIterations(patULong i) ;
  void setRunTime(patString t) ;
  void setInitLL(patReal l) ;
  void addStatistic(patString name, patReal value) ;
  void setSimulationResults(map<patString,vector<patReal> >* s) ;
  void printEstimatedParameters(patString filename, patError*& err) ;
 private:
  void compute(patError*& err) ;
  bioRawResults* theRawResults ;
  bioBayesianResults* theBayesianResults ;
  patVariables stdErr ;
  patVariables robustStdErr ;
  patULong nbrDraws ;
  patULong nbrParameters ;
  patULong sampleSize ;
  patReal initLoglikelihood ;
  patString diagnostic ;
  patULong iterations ;
  patString runTime ;
  patString sampleFile ;
  map<patString, patReal> statistics ;
  map<patString,vector<patReal> >* simulatedValues ;
  
};

#endif
