//-*-c++-*------------------------------------------------------------
//
// File name : patAntithetic.cc
// Author :    Michel Bierlaire
// Date :      Thu Jun  3 07:36:20 2010
//
//--------------------------------------------------------------------

#include "patAntithetic.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patDisplay.h"
#include "patFileNames.h"

patAntithetic::patAntithetic(patRandomNumberGenerator* draws, 
			     patULong R,
			     patULong s,
			     patBoolean dumpDrawsOnFile,
			     patError*& err) :
  patRandomNumberGenerator(dumpDrawsOnFile),
  rndNumberGenerator(draws),
  logFile(NULL),
  nbrOfDraws(R),
  nbrOfSeries(s),
  theDraws(s,vector<patReal>(ceil(R/2.0))),
  first(patTRUE),
  currentValue(patBadId),
  currentSerie(patBadId) {

  if (rndNumberGenerator == NULL) {
    err = new patErrNullPointer("patRandomNumberGenerator") ;
    WARNING(err->describe()) ;
    return ;
  }

  if (!draws->isSymmetric()) {
    err = new patErrMiscError("Antithetic draws can be generated automatically only for symmetric draws. A specific implementation must be done explicitly for this generator.") ;
    WARNING(err->describe()) ;
  }

  if (dumpDrawsOnFile) {
    logFile = new ofstream(patFileNames::the()->getUnifDrawLogFile(err).c_str()) ;  }
  
}

patAntithetic::~patAntithetic() {
  
}

patReal patAntithetic::getNextValue(patError*& err) {
  if (currentValue == patBadId || currentSerie == patBadId) {
    generate(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
  }

  patReal result ;
  if (first) {
    result = theDraws[currentSerie][currentValue] ;
  }
  else {
    result = -theDraws[currentSerie][currentValue] ;
  }

  ++currentValue ;
  if (currentValue == theDraws[currentSerie].size()) {
    if (first) {
      currentValue = 0 ;
      first = patFALSE;
    }
    else {
      // Next serie
      ++currentSerie ;
      if (currentSerie == nbrOfSeries) {
	currentSerie = patBadId ;
	currentValue = patBadId ;
      }
      else {
	currentValue = 0 ;
	first = patTRUE;
      }
    }
  }
  if (logFile != NULL) {
    *logFile << result << endl ;
  }
  return result ;
  
}


patReal patAntithetic::getZeroOneDraw(patError*& err) {
  if (rndNumberGenerator == NULL) {
    err = new patErrNullPointer("patRandomNumberGenerator") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  patReal result = rndNumberGenerator->getZeroOneDraw(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal() ;
  }
  return result ;
}
patBoolean patAntithetic::isSymmetric() const {
  return patTRUE ;
}


void patAntithetic::generate(patError*& err) {
  
  currentValue = 0 ;
  currentSerie = 0 ;
  for (patULong j = 0 ; j < nbrOfSeries ; ++j) {
    for (patULong i = 0 ; i < theDraws[j].size() ; ++i) {
      theDraws[j][i] = rndNumberGenerator->getNextValue(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
  }

}

patBoolean patAntithetic::isNormal() const {
  return patFALSE ;
}

