from biogeme import *

## Simply computes the log of the probability
# \param prob An <a
# href="http://biogeme.epfl.ch/expressions.html">expression</a>
# providing the value of the probability.
# \return The logarithm of the probability.
#
# \code
# def loglikelihood(prob):
#     loglikelihood = log(prob)
#     return loglikelihood
# \endcode 
def loglikelihood(prob):
    loglikelihood = log(prob)
    return loglikelihood

## Compute a simulated loglikelihood function
# \param prob An <a
# href="http://biogeme.epfl.ch/expressions.html">expression</a>
# providing the value of the probability. Although it is not formally
# necessary, the expression should contain one or more <a
# href="http://biogeme.epfl.ch/expressions.php#draws"
# target="_blank">random variables</a> of a given distribution, and
# therefore write \f[ P(i|\xi_1,\ldots,\xi_L)\f] 
# \return The simulated
# loglikelihood, given by 
# \f[ \ln\left(\sum_{r=1}^R P(i|\xi^r_1,\ldots,\xi^r_L) \right)\f]
# where \f$R\f$ is the number of draws, and \f$\xi_j^r\f$ is the
# <em>r</em>th draw of the random variable \f$\xi_j\f$.
#
# \code
# def mixedloglikelihood(prob):
#     drawIterator('drawIter')
#     l = Sum(prob,'drawIter')
#     return log(l)
# \endcode
def mixedloglikelihood(prob):
    drawIterator('drawIter')
    l = Sum(prob,'drawIter')
    return log(l)

## Computes likelihood function of a regression model.
# \param meas An <a href="http://biogeme.epfl.ch/expressions.html">expression</a> providing the value \f$y\f$ of the measure for the current observation.
# \param model An <a href="http://biogeme.epfl.ch/expressions.html">expression</a> providing the output \f$m\f$ of the model for the current observation.
# \param sigma An <a href="http://biogeme.epfl.ch/expressions.html">expression</a>
# (typically, a <a href="http://biogeme.epfl.ch/expressions.php#parameter">parameter</a>)
# providing the standard error \f$\sigma\f$ of the error term.
# \return The likelihood of the regression, assuming a normal
# distribution, that is
# \f[
#  \frac{1}{\sigma} \phi\left( \frac{y-m}{\sigma} \right) 
# \f]
# where \f$ \phi(\cdot)\f$ is the pdf of the normal distribution.
#
# \code
# def likelihoodregression(meas,model,sigma):
#     t = (meas - model) / sigma
#     f = bioNormalPdf(t) / sigma
#     return f
# \endcode
def likelihoodregression(meas,model,sigma):
    t = (meas - model) / sigma
    f = bioNormalPdf(t) / sigma
    return f


